package com.hh.arome.utlis.log;

import static com.hh.arome.common.Constants.LogcatValue.LINE;
import static com.hh.arome.common.Constants.LogcatValue.LOG_NODE_APPLET;
import static com.hh.arome.common.Constants.LogcatValue.LOG_NODE_MICROPHONE;
import static com.hh.arome.common.Constants.LogcatValue.NET_LOGCAT;
import static com.hh.arome.common.Constants.LogcatValue.NODE;
import static com.hh.arome.common.Constants.LogcatValue.WARP;
import static com.hh.arome.common.Constants.Value.TOKEN;

import com.google.gson.JsonObject;
import com.hh.arome.bean.LogcatAddRequestBean;
import com.hh.arome.mvp.presenter.LogcatAddPresenter;
import com.hh.arome.utlis.device.DeviceUtils;

/**
 * @Description:
 * @author: Xia WeiHao
 * @date: 2024/6/25
 */
public class AppNetLogUtils {


    public static void sendLogs(LogcatAddPresenter presenter, boolean isNetAvailable,
                          String action, JsonObject jsonObject, String defaultLog) {
        try {
            // 日志开始----------------begin
            // 日志行数——对应web端"日志标签"
            String logLine = (new Throwable().getStackTrace())[0].toString();
            // 日志节点
            String logNode = action + "--回传处理结果给小程序--";
            // 日志内容
            StringBuilder logContentSb = new StringBuilder();
            logContentSb.append(logNode).append(defaultLog).append(WARP)
                    .append("jsonObject = ").append(jsonObject);

            // 开始保存本地日志
            AppLogUtils.saveLog(LogType.Logcat, new StringBuilder()
                    .append(LINE)
                    .append(logLine).append(WARP)
                    .append(NODE)
                    .append(logContentSb)
                    .toString()
            );

            // 开始网络在线日志
            LogcatAddRequestBean logBean = new LogcatAddRequestBean();
            logBean.setLogType(NET_LOGCAT);
            // 对应web端"日志标签"
            logBean.setLogTag(logLine + WARP + LOG_NODE_APPLET + action);
            logBean.setLogContent(logContentSb.toString());
            requestLogcatAdd(presenter,isNetAvailable,logBean);
            // 日志开始----------------end

            // 打印编译器输出日志
            AppLogUtils.e(logContentSb);

        } catch (Throwable t) {
            AppLogUtils.e("onCalled--t" + t);
        }
    }


    public static void sendLogs(LogcatAddPresenter presenter, boolean isNetAvailable,
                                String action, String errLog) {
        try {
            // 日志开始----------------begin
            // 日志行数——对应web端"日志标签"
            String logLine = (new Throwable().getStackTrace())[0].toString();
            // 日志节点
            String logNode = action + "--错误日志--";
            // 日志内容
            StringBuilder logContentSb = new StringBuilder();
            logContentSb.append(logNode).append(errLog);


            // 开始保存本地日志
            AppLogUtils.saveLog(LogType.Logcat, new StringBuilder()
                    .append(LINE)
                    .append(logLine).append(WARP)
                    .append(NODE)
                    .append(logContentSb)
                    .toString()
            );

            // 开始网络在线日志
            LogcatAddRequestBean logBean = new LogcatAddRequestBean();
            logBean.setLogType(NET_LOGCAT);
            // 对应web端"日志标签"
            logBean.setLogTag(logLine + WARP + LOG_NODE_APPLET + action);
            logBean.setLogContent(logContentSb.toString());
            requestLogcatAdd(presenter,isNetAvailable,logBean);
            // 日志开始----------------end

            // 打印编译器输出日志
            AppLogUtils.e(logContentSb);

        } catch (Throwable t) {
            AppLogUtils.e("onCalled--t" + t);
        }
    }

    private static void requestLogcatAdd(LogcatAddPresenter presenter, boolean isNetAvailable,LogcatAddRequestBean bean) {
        if(presenter==null){
            return;
        }
        bean.setToken(TOKEN);
        bean.setSn(DeviceUtils.getDeviceSN());
        if (isNetAvailable) {
            presenter.requestLogcatAdd(bean);
        }

    }

}
